# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import List, Any, Callable
from rx.subjects import ReplaySubject  # type: ignore
from rx.core import ObservableBase  # type: ignore

from .call import call, call_async, call_sync
from .motion_lib_exception import MotionLibException
from .events import events

from .device import Device
from .response import Response
from .unknown_response_event import UnknownResponseEvent
from .alert_event import AlertEvent
from .protobufs import main_pb2


class Connection:
    """
    Class representing access to particular connection (serial port, TCP connection).
    """

    @property
    def unknown_response(self) -> ObservableBase:
        """
        Event invoked when a response from a device cannot be matched to any known request.
        """
        return self._unknown_response

    @property
    def alert(self) -> ObservableBase:
        """
        Event invoked when an alert is received from a device.
        """
        return self._alert

    @property
    def disconnected(self) -> ObservableBase:
        """
        Event invoked when connection is interrupted or closed.
        """
        return self._disconnected

    @property
    def interface_id(self) -> int:
        """
        The interface ID identifies this Connection instance with the underlying library.
        """
        return self._interface_id

    def __init__(self, interface_id: int):
        self._interface_id = interface_id
        self.__setup_events()

    @staticmethod
    def open_serial_port(
            port_name: str,
            baud_rate: int = 0
    ) -> 'Connection':
        """
        Opens a serial port.

        Args:
            port_name: Name of the port to open.
            baud_rate: Optional baud rate (defaults to 115200).

        Returns:
            An object representing the port.
        """
        request = main_pb2.OpenInterfaceRequest()
        request.interface_type = main_pb2.OpenInterfaceRequest.SERIAL_PORT
        request.port_name = port_name
        request.baud_rate = baud_rate
        response = main_pb2.OpenInterfaceResponse()
        call("interface/open", request, response)
        return Connection(response.interface_id)

    @staticmethod
    async def open_serial_port_async(
            port_name: str,
            baud_rate: int = 0
    ) -> 'Connection':
        """
        Opens a serial port.

        Args:
            port_name: Name of the port to open.
            baud_rate: Optional baud rate (defaults to 115200).

        Returns:
            An object representing the port.
        """
        request = main_pb2.OpenInterfaceRequest()
        request.interface_type = main_pb2.OpenInterfaceRequest.SERIAL_PORT
        request.port_name = port_name
        request.baud_rate = baud_rate
        response = main_pb2.OpenInterfaceResponse()
        await call_async("interface/open", request, response)
        return Connection(response.interface_id)

    @staticmethod
    def open_tcp(
            host_name: str,
            port: int
    ) -> 'Connection':
        """
        Opens a TCP connection.

        Args:
            host_name: Hostname or IP address.
            port: Port number.

        Returns:
            An object representing the connection.
        """
        request = main_pb2.OpenInterfaceRequest()
        request.interface_type = main_pb2.OpenInterfaceRequest.TCP
        request.host_name = host_name
        request.port = port
        response = main_pb2.OpenInterfaceResponse()
        call("interface/open", request, response)
        return Connection(response.interface_id)

    @staticmethod
    async def open_tcp_async(
            host_name: str,
            port: int
    ) -> 'Connection':
        """
        Opens a TCP connection.

        Args:
            host_name: Hostname or IP address.
            port: Port number.

        Returns:
            An object representing the connection.
        """
        request = main_pb2.OpenInterfaceRequest()
        request.interface_type = main_pb2.OpenInterfaceRequest.TCP
        request.host_name = host_name
        request.port = port
        response = main_pb2.OpenInterfaceResponse()
        await call_async("interface/open", request, response)
        return Connection(response.interface_id)

    def generic_command(
            self,
            command: str,
            device: int = 0,
            axis: int = 0,
            check_errors: bool = True
    ) -> Response:
        """
        Sends a generic ASCII command to this port.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command to send.
            device: Optional device address to send the command to.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw exception when device rejects the command.

        Returns:
            A response to the command.
        """
        request = main_pb2.GenericCommandRequest()
        request.interface_id = self.interface_id
        request.command = command
        request.device = device
        request.axis = axis
        request.check_errors = check_errors
        response = main_pb2.GenericCommandResponse()
        call("interface/generic_command", request, response)
        return Response.from_protobuf(response)

    async def generic_command_async(
            self,
            command: str,
            device: int = 0,
            axis: int = 0,
            check_errors: bool = True
    ) -> Response:
        """
        Sends a generic ASCII command to this port.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command to send.
            device: Optional device address to send the command to.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw exception when device rejects the command.

        Returns:
            A response to the command.
        """
        request = main_pb2.GenericCommandRequest()
        request.interface_id = self.interface_id
        request.command = command
        request.device = device
        request.axis = axis
        request.check_errors = check_errors
        response = main_pb2.GenericCommandResponse()
        await call_async("interface/generic_command", request, response)
        return Response.from_protobuf(response)

    def generic_command_no_response(
            self,
            command: str,
            device: int = 0,
            axis: int = 0
    ) -> None:
        """
        Sends a generic ASCII command to this port without expecting a response and without adding a message ID.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command to send.
            device: Optional device address to send the command to.
            axis: Optional axis number to send the command to.
        """
        request = main_pb2.GenericCommandRequest()
        request.interface_id = self.interface_id
        request.command = command
        request.device = device
        request.axis = axis
        call("interface/generic_command_no_response", request)

    async def generic_command_no_response_async(
            self,
            command: str,
            device: int = 0,
            axis: int = 0
    ) -> None:
        """
        Sends a generic ASCII command to this port without expecting a response and without adding a message ID.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command to send.
            device: Optional device address to send the command to.
            axis: Optional axis number to send the command to.
        """
        request = main_pb2.GenericCommandRequest()
        request.interface_id = self.interface_id
        request.command = command
        request.device = device
        request.axis = axis
        await call_async("interface/generic_command_no_response", request)

    def generic_command_multi_response(
            self,
            command: str,
            device: int = 0,
            axis: int = 0,
            check_errors: bool = True
    ) -> List[Response]:
        """
        Sends a generic ASCII command to this port and expect multiple responses,
        either from one device or from many devices.
        Responses are returned in order of arrival.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command to send.
            device: Optional device address to send the command to.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw exception when device rejects the command.

        Returns:
            All responses to the command.
        """
        request = main_pb2.GenericCommandRequest()
        request.interface_id = self.interface_id
        request.command = command
        request.device = device
        request.axis = axis
        request.check_errors = check_errors
        response = main_pb2.GenericCommandResponseCollection()
        call("interface/generic_command_multi_response", request, response)
        return [Response.from_protobuf(resp) for resp in response.responses]

    async def generic_command_multi_response_async(
            self,
            command: str,
            device: int = 0,
            axis: int = 0,
            check_errors: bool = True
    ) -> List[Response]:
        """
        Sends a generic ASCII command to this port and expect multiple responses,
        either from one device or from many devices.
        Responses are returned in order of arrival.
        For more information refer to: [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_commands).

        Args:
            command: Command to send.
            device: Optional device address to send the command to.
            axis: Optional axis number to send the command to.
            check_errors: Controls whether to throw exception when device rejects the command.

        Returns:
            All responses to the command.
        """
        request = main_pb2.GenericCommandRequest()
        request.interface_id = self.interface_id
        request.command = command
        request.device = device
        request.axis = axis
        request.check_errors = check_errors
        response = main_pb2.GenericCommandResponseCollection()
        await call_async("interface/generic_command_multi_response", request, response)
        return [Response.from_protobuf(resp) for resp in response.responses]

    def reset_ids(
            self
    ) -> None:
        """
        Resets ASCII protocol message IDs. Only for testing purposes.
        """
        request = main_pb2.ResetIdsRequest()
        request.interface_id = self.interface_id
        call_sync("interface/reset_ids", request)

    def close(
            self
    ) -> None:
        """
        Close the serial port.
        """
        request = main_pb2.CloseInterfaceRequest()
        request.interface_id = self.interface_id
        call("interface/close", request)

    async def close_async(
            self
    ) -> None:
        """
        Close the serial port.
        """
        request = main_pb2.CloseInterfaceRequest()
        request.interface_id = self.interface_id
        await call_async("interface/close", request)

    def device(
            self,
            device_address: int
    ) -> Device:
        """
        Gets Device class instance which allows you to control particular device on this connection.

        Args:
            device_address: Address of device intended to control. Address is configured for each device.

        Returns:
            Device instance.
        """
        if device_address <= 0:
            raise ValueError('Invalid value, physical devices are numbered from 1.')

        return Device(self, device_address)

    def renumber_devices(
            self,
            first_address: int = 1
    ) -> int:
        """
        Renumbers devices present on this connection. After renumbering, devices need to be identified again.

        Args:
            first_address: This is the address that the device closest to the computer is given.
                Remaining devices are numbered consecutively.

        Returns:
            Total number of devices that responded to the renumber.
        """
        if first_address <= 0:
            raise ValueError('Invalid value, device addresses are numbered from 1.')

        request = main_pb2.DeviceRenumberRequest()
        request.interface_id = self.interface_id
        request.first_address = first_address
        response = main_pb2.DeviceRenumberResponse()
        call("device/renumber", request, response)
        return response.number_devices

    async def renumber_devices_async(
            self,
            first_address: int = 1
    ) -> int:
        """
        Renumbers devices present on this connection. After renumbering, devices need to be identified again.

        Args:
            first_address: This is the address that the device closest to the computer is given.
                Remaining devices are numbered consecutively.

        Returns:
            Total number of devices that responded to the renumber.
        """
        if first_address <= 0:
            raise ValueError('Invalid value, device addresses are numbered from 1.')

        request = main_pb2.DeviceRenumberRequest()
        request.interface_id = self.interface_id
        request.first_address = first_address
        response = main_pb2.DeviceRenumberResponse()
        await call_async("device/renumber", request, response)
        return response.number_devices

    def detect_devices(
            self,
            identify_devices: bool = True
    ) -> List[Device]:
        """
        Attempts to detect any devices present on this connection.

        Args:
            identify_devices: Determines whether device identification should be performed as well.

        Returns:
            Array of detected devices.
        """
        request = main_pb2.DeviceDetectRequest()
        request.interface_id = self.interface_id
        request.identify_devices = identify_devices
        response = main_pb2.DeviceDetectResponse()
        call("device/detect", request, response)
        return list(map(self.device, response.devices))

    async def detect_devices_async(
            self,
            identify_devices: bool = True
    ) -> List[Device]:
        """
        Attempts to detect any devices present on this connection.

        Args:
            identify_devices: Determines whether device identification should be performed as well.

        Returns:
            Array of detected devices.
        """
        request = main_pb2.DeviceDetectRequest()
        request.interface_id = self.interface_id
        request.identify_devices = identify_devices
        response = main_pb2.DeviceDetectResponse()
        await call_async("device/detect", request, response)
        return list(map(self.device, response.devices))

    def __repr__(
            self
    ) -> str:
        """
        Returns a string that represents the connection.

        Returns:
            A string that represents the connection.
        """
        request = main_pb2.ToStringRequest()
        request.interface_id = self.interface_id
        response = main_pb2.ToStringResponse()
        call_sync("interface/to_string", request, response)
        return response.to_str

    def __enter__(self) -> 'Connection':
        """ __enter__ """
        return self

    def __exit__(self, _type: Any, _value: Any, _traceback: Any) -> None:
        """ __exit__ """
        self.close()

    def __filter_event(self, event_name: str) -> Callable[[Any], bool]:
        def filter_event(event: Any) -> bool:
            return event[0] == event_name and event[1].interface_id == self._interface_id  # type: ignore
        return filter_event

    def __setup_events(self) -> None:
        self._disconnected = ReplaySubject()  # terminates all the events

        self._unknown_response = events.take_until(self.disconnected)\
            .filter(self.__filter_event('interface/unknown_response'))\
            .map(lambda ev: UnknownResponseEvent.from_protobuf(ev[1]))\

        self._alert = events.take_until(self.disconnected)\
            .filter(self.__filter_event('interface/alert'))\
            .map(lambda ev: AlertEvent.from_protobuf(ev[1]))\

        events.filter(self.__filter_event('interface/disconnected'))\
            .take(1)\
            .map(lambda ev: MotionLibException(ev[1].error_type, ev[1].error_message))\
            .subscribe(self.disconnected)



from .connection import Connection
from .device import Device
from .axis import Axis
from .all_axes import AllAxes
from .motion_lib_exception import MotionLibException
from .units import Units
from .axis_settings import AxisSettings
from .device_settings import DeviceSettings
from .warnings import Warnings
from .warning_flags import WarningFlags
from .library import Library
from .log_output_mode import LogOutputMode
from .protobufs import main_pb2 as protobufs
from .unknown_response_event import UnknownResponseEvent
from .device_identity import DeviceIdentity
from .axis_identity import AxisIdentity
from .message_type import MessageType
from .axis_type import AxisType
from .tools import Tools
from .device_db_source_type import DeviceDbSourceType
from .measurement import Measurement
from .motion_lib_error_type import MotionLibErrorType
__version__ = "0.0.21"



from .motion_lib_error_type import MotionLibErrorType


class MotionLibException(Exception):

    @property
    def error_type(self) -> MotionLibErrorType:
        """
        Specific error returned by the native library.
        """
        return self._error_type

    @property
    def message(self) -> str:
        """
        Error message of the exception.
        """
        return self._message

    def __init__(self, error_type: int, message: str):
        self._message = message
        self._error_type = MotionLibErrorType(error_type)
        Exception.__init__(self, "{}: {}".format(self._error_type.name, self._message))



from enum import Enum


class MotionLibErrorType(Enum):
    """
    Enumeration of various errors that may occur during usage of the library.
    """

    REQUEST_TIMEOUT = 0
    CONNECTION_CLOSED = 1
    INVALID_ARGUMENT = 2
    OUT_OF_REQUEST_IDS = 3
    INVALID_PACKET = 4
    CONNECTION_FAILED = 5
    UNKNOWN_REQUEST = 6
    COMMAND_FAILED = 7
    DEVICE_DB_FAILED = 8
    INVALID_DATA = 9
    DEVICE_NOT_IDENTIFIED = 10
    CONVERSION_FAILED = 11
    DEVICE_NUMBER_CONFLICT = 12
    NO_DEVICE_FOUND = 13
    MOVEMENT_INTERRUPTED = 14
    MOVEMENT_FAILED = 15
    IO_ERROR = 16
    INVALID_RESPONSE = 17
    NOT_SUPPORTED = 18
    DEVICE_FAILED = 19
    OS_ERROR = 20



#  This file is generated from the Zaber device database. Do not manually edit this file.

from enum import Enum


class Units(Enum):
    Native = ''

    Length_metres = 'Length:metres'
    Length_centimetres = 'Length:centimetres'
    Length_millimetres = 'Length:millimetres'
    Length_micrometres = 'Length:micrometres'
    Length_nanometres = 'Length:nanometres'
    Length_inches = 'Length:inches'

    Velocity_metres_per_second = 'Velocity:metres per second'
    Velocity_centimetres_per_second = 'Velocity:centimetres per second'
    Velocity_millimetres_per_second = 'Velocity:millimetres per second'
    Velocity_micrometres_per_second = 'Velocity:micrometres per second'
    Velocity_nanometres_per_second = 'Velocity:nanometres per second'
    Velocity_inches_per_second = 'Velocity:inches per second'

    Acceleration_metres_per_second_squared = 'Acceleration:metres per second squared'
    Acceleration_centimetres_per_second_squared = 'Acceleration:centimetres per second squared'
    Acceleration_millimetres_per_second_squared = 'Acceleration:millimetres per second squared'
    Acceleration_micrometres_per_second_squared = 'Acceleration:micrometres per second squared'
    Acceleration_nanometres_per_second_squared = 'Acceleration:nanometres per second squared'
    Acceleration_inches_per_second_squared = 'Acceleration:inches per second squared'

    Angle_degrees = 'Angle:degrees'
    Angle_radians = 'Angle:radians'

    AngularVelocity_degrees_per_second = 'Angular Velocity:degrees per second'
    AngularVelocity_radians_per_second = 'Angular Velocity:radians per second'

    AngularAcceleration_degrees_per_second_squared = 'Angular Acceleration:degrees per second squared'
    AngularAcceleration_radians_per_second_squared = 'Angular Acceleration:radians per second squared'

    ACElectricCurrent_amperes_peak = 'AC Electric Current:amperes peak'
    ACElectricCurrent_amperes_rms = 'AC Electric Current:amperes RMS'

    Percent_percent = 'Percent:percent'

    DCElectricCurrent_amperes = 'DC Electric Current:amperes'

    Force_newtons = 'Force:newtons'
    Force_millinewtons = 'Force:millinewtons'
    Force_pounds_force = 'Force:pounds-force'
    Force_kilonewtons = 'Force:kilonewtons'

    Time_seconds = 'Time:seconds'
    Time_milliseconds = 'Time:milliseconds'
    Time_microseconds = 'Time:microseconds'

    Torque_newton_metres = 'Torque:newton metres'
    Torque_newton_centimetres = 'Torque:newton centimetres'
    Torque_pound_force_feet = 'Torque:pound-force-feet'
    Torque_ounce_force_inches = 'Torque:ounce-force-inches'

    Inertia_grams = 'Inertia:grams'
    Inertia_kilograms = 'Inertia:kilograms'
    Inertia_milligrams = 'Inertia:milligrams'
    Inertia_pounds = 'Inertia:pounds'
    Inertia_ounces = 'Inertia:ounces'

    RotationalInertia_gram_square_metre = 'Rotational Inertia:gram-square metre'
    RotationalInertia_kilogram_square_metre = 'Rotational Inertia:kilogram-square metre'
    RotationalInertia_pound_square_feet = 'Rotational Inertia:pound-square-feet'

    ForceConstant_newtons_per_amp = 'Force Constant:newtons per amp'
    ForceConstant_millinewtons_per_amp = 'Force Constant:millinewtons per amp'
    ForceConstant_kilonewtons_per_amp = 'Force Constant:kilonewtons per amp'
    ForceConstant_pounds_force_per_amp = 'Force Constant:pounds-force per amp'

    TorqueConstant_newton_metres_per_amp = 'Torque Constant:newton metres per amp'
    TorqueConstant_millinewton_metres_per_amp = 'Torque Constant:millinewton metres per amp'
    TorqueConstant_kilonewton_metres_per_amp = 'Torque Constant:kilonewton metres per amp'
    TorqueConstant_pound_force_feet_per_amp = 'Torque Constant:pound-force-feet per amp'



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
from google.protobuf.descriptor import (
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    List as typing___List,
    Optional as typing___Optional,
    Text as typing___Text,
    Tuple as typing___Tuple,
    cast as typing___cast,
)


class MessageType(int):
    DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
    @classmethod
    def Name(cls, number: int) -> str: ...
    @classmethod
    def Value(cls, name: str) -> MessageType: ...
    @classmethod
    def keys(cls) -> typing___List[str]: ...
    @classmethod
    def values(cls) -> typing___List[MessageType]: ...
    @classmethod
    def items(cls) -> typing___List[typing___Tuple[str, MessageType]]: ...
REPLY = typing___cast(MessageType, 0)
INFO = typing___cast(MessageType, 1)
ALERT = typing___cast(MessageType, 2)

class Errors(int):
    DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
    @classmethod
    def Name(cls, number: int) -> str: ...
    @classmethod
    def Value(cls, name: str) -> Errors: ...
    @classmethod
    def keys(cls) -> typing___List[str]: ...
    @classmethod
    def values(cls) -> typing___List[Errors]: ...
    @classmethod
    def items(cls) -> typing___List[typing___Tuple[str, Errors]]: ...
REQUEST_TIMEOUT = typing___cast(Errors, 0)
CONNECTION_CLOSED = typing___cast(Errors, 1)
INVALID_ARGUMENT = typing___cast(Errors, 2)
OUT_OF_REQUEST_IDS = typing___cast(Errors, 3)
INVALID_PACKET = typing___cast(Errors, 4)
CONNECTION_FAILED = typing___cast(Errors, 5)
UNKNOWN_REQUEST = typing___cast(Errors, 6)
COMMAND_FAILED = typing___cast(Errors, 7)
DEVICE_DB_FAILED = typing___cast(Errors, 8)
INVALID_DATA = typing___cast(Errors, 9)
DEVICE_NOT_IDENTIFIED = typing___cast(Errors, 10)
CONVERSION_FAILED = typing___cast(Errors, 11)
DEVICE_NUMBER_CONFLICT = typing___cast(Errors, 12)
NO_DEVICE_FOUND = typing___cast(Errors, 13)
MOVEMENT_INTERRUPTED = typing___cast(Errors, 14)
MOVEMENT_FAILED = typing___cast(Errors, 15)
IO_ERROR = typing___cast(Errors, 16)
INVALID_RESPONSE = typing___cast(Errors, 17)
NOT_SUPPORTED = typing___cast(Errors, 18)
DEVICE_FAILED = typing___cast(Errors, 19)
OS_ERROR = typing___cast(Errors, 20)

class Request(google___protobuf___message___Message):
    request = ... # type: typing___Text

    def __init__(self,
        request : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> Request: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class Response(google___protobuf___message___Message):
    class ResponseType(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> Response.ResponseType: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[Response.ResponseType]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, Response.ResponseType]]: ...
    OK = typing___cast(ResponseType, 0)
    ERROR = typing___cast(ResponseType, 1)

    response = ... # type: Response.ResponseType
    error_type = ... # type: Errors
    error_message = ... # type: typing___Text

    def __init__(self,
        response : typing___Optional[Response.ResponseType] = None,
        error_type : typing___Optional[Errors] = None,
        error_message : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> Response: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class Event(google___protobuf___message___Message):
    event = ... # type: typing___Text

    def __init__(self,
        event : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> Event: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class EmptyRequest(google___protobuf___message___Message):

    def __init__(self,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> EmptyRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class OpenInterfaceRequest(google___protobuf___message___Message):
    class InterfaceType(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> OpenInterfaceRequest.InterfaceType: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[OpenInterfaceRequest.InterfaceType]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, OpenInterfaceRequest.InterfaceType]]: ...
    SERIAL_PORT = typing___cast(InterfaceType, 0)
    TCP = typing___cast(InterfaceType, 1)

    interface_type = ... # type: OpenInterfaceRequest.InterfaceType
    port_name = ... # type: typing___Text
    baud_rate = ... # type: int
    host_name = ... # type: typing___Text
    port = ... # type: int

    def __init__(self,
        interface_type : typing___Optional[OpenInterfaceRequest.InterfaceType] = None,
        port_name : typing___Optional[typing___Text] = None,
        baud_rate : typing___Optional[int] = None,
        host_name : typing___Optional[typing___Text] = None,
        port : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> OpenInterfaceRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class OpenInterfaceResponse(google___protobuf___message___Message):
    interface_id = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> OpenInterfaceResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class CloseInterfaceRequest(google___protobuf___message___Message):
    interface_id = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CloseInterfaceRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class ToStringRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ToStringRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class ToStringResponse(google___protobuf___message___Message):
    to_str = ... # type: typing___Text

    def __init__(self,
        to_str : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ToStringResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class GenericCommandRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    command = ... # type: typing___Text
    check_errors = ... # type: bool

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        command : typing___Optional[typing___Text] = None,
        check_errors : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> GenericCommandRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class GenericCommandResponse(google___protobuf___message___Message):
    device_address = ... # type: int
    axis_number = ... # type: int
    reply_flag = ... # type: typing___Text
    status = ... # type: typing___Text
    warning_flag = ... # type: typing___Text
    data = ... # type: typing___Text
    message_type = ... # type: MessageType

    def __init__(self,
        device_address : typing___Optional[int] = None,
        axis_number : typing___Optional[int] = None,
        reply_flag : typing___Optional[typing___Text] = None,
        status : typing___Optional[typing___Text] = None,
        warning_flag : typing___Optional[typing___Text] = None,
        data : typing___Optional[typing___Text] = None,
        message_type : typing___Optional[MessageType] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> GenericCommandResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class GenericCommandResponseCollection(google___protobuf___message___Message):

    @property
    def responses(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[GenericCommandResponse]: ...

    def __init__(self,
        responses : typing___Optional[typing___Iterable[GenericCommandResponse]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> GenericCommandResponseCollection: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class UnknownResponseEvent(google___protobuf___message___Message):
    interface_id = ... # type: int
    device_address = ... # type: int
    axis_number = ... # type: int
    reply_flag = ... # type: typing___Text
    status = ... # type: typing___Text
    warning_flag = ... # type: typing___Text
    data = ... # type: typing___Text
    message_type = ... # type: MessageType

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device_address : typing___Optional[int] = None,
        axis_number : typing___Optional[int] = None,
        reply_flag : typing___Optional[typing___Text] = None,
        status : typing___Optional[typing___Text] = None,
        warning_flag : typing___Optional[typing___Text] = None,
        data : typing___Optional[typing___Text] = None,
        message_type : typing___Optional[MessageType] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> UnknownResponseEvent: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class AlertEvent(google___protobuf___message___Message):
    interface_id = ... # type: int
    device_address = ... # type: int
    axis_number = ... # type: int
    status = ... # type: typing___Text
    warning_flag = ... # type: typing___Text
    data = ... # type: typing___Text

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device_address : typing___Optional[int] = None,
        axis_number : typing___Optional[int] = None,
        status : typing___Optional[typing___Text] = None,
        warning_flag : typing___Optional[typing___Text] = None,
        data : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> AlertEvent: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DisconnectedEvent(google___protobuf___message___Message):
    interface_id = ... # type: int
    error_type = ... # type: Errors
    error_message = ... # type: typing___Text

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        error_type : typing___Optional[Errors] = None,
        error_message : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DisconnectedEvent: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class TestRequest(google___protobuf___message___Message):
    return_error = ... # type: bool
    data_ping = ... # type: typing___Text

    def __init__(self,
        return_error : typing___Optional[bool] = None,
        data_ping : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> TestRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class TestResponse(google___protobuf___message___Message):
    data_pong = ... # type: typing___Text

    def __init__(self,
        data_pong : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> TestResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class TestEvent(google___protobuf___message___Message):
    data = ... # type: typing___Text

    def __init__(self,
        data : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> TestEvent: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceRenumberRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    first_address = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        first_address : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceRenumberRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceRenumberResponse(google___protobuf___message___Message):
    number_devices = ... # type: int

    def __init__(self,
        number_devices : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceRenumberResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceDetectRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    identify_devices = ... # type: bool

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        identify_devices : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceDetectRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceDetectResponse(google___protobuf___message___Message):
    devices = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[int]

    def __init__(self,
        devices : typing___Optional[typing___Iterable[int]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceDetectResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceHomeRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    wait_until_idle = ... # type: bool

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        wait_until_idle : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceHomeRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceWaitUntilIdleRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    throw_error_on_fault = ... # type: bool

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        throw_error_on_fault : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceWaitUntilIdleRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceMoveRequest(google___protobuf___message___Message):
    class MoveType(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> DeviceMoveRequest.MoveType: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[DeviceMoveRequest.MoveType]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, DeviceMoveRequest.MoveType]]: ...
    ABS = typing___cast(MoveType, 0)
    REL = typing___cast(MoveType, 1)
    VEL = typing___cast(MoveType, 2)

    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    wait_until_idle = ... # type: bool
    type = ... # type: DeviceMoveRequest.MoveType
    arg = ... # type: float
    unit = ... # type: typing___Text

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        wait_until_idle : typing___Optional[bool] = None,
        type : typing___Optional[DeviceMoveRequest.MoveType] = None,
        arg : typing___Optional[float] = None,
        unit : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceMoveRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceStopRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    wait_until_idle = ... # type: bool

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        wait_until_idle : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceStopRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceIdentifyRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceIdentifyRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class AxisIdentity(google___protobuf___message___Message):
    class AxisType(int):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        @classmethod
        def Name(cls, number: int) -> str: ...
        @classmethod
        def Value(cls, name: str) -> AxisIdentity.AxisType: ...
        @classmethod
        def keys(cls) -> typing___List[str]: ...
        @classmethod
        def values(cls) -> typing___List[AxisIdentity.AxisType]: ...
        @classmethod
        def items(cls) -> typing___List[typing___Tuple[str, AxisIdentity.AxisType]]: ...
    UNKNOWN = typing___cast(AxisType, 0)
    LINEAR = typing___cast(AxisType, 1)
    ROTARY = typing___cast(AxisType, 2)

    is_peripheral = ... # type: bool
    peripheral_id = ... # type: int
    name = ... # type: typing___Text
    axis_type = ... # type: AxisIdentity.AxisType

    def __init__(self,
        is_peripheral : typing___Optional[bool] = None,
        peripheral_id : typing___Optional[int] = None,
        name : typing___Optional[typing___Text] = None,
        axis_type : typing___Optional[AxisIdentity.AxisType] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> AxisIdentity: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceIdentity(google___protobuf___message___Message):
    device_id = ... # type: int
    serial_number = ... # type: int
    name = ... # type: typing___Text
    axis_count = ... # type: int

    @property
    def firmware_version(self) -> FirmwareVersion: ...

    @property
    def axes(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[AxisIdentity]: ...

    def __init__(self,
        device_id : typing___Optional[int] = None,
        serial_number : typing___Optional[int] = None,
        name : typing___Optional[typing___Text] = None,
        axis_count : typing___Optional[int] = None,
        firmware_version : typing___Optional[FirmwareVersion] = None,
        axes : typing___Optional[typing___Iterable[AxisIdentity]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceIdentity: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceGetIdentityRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceGetIdentityRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceGetIdentityResponse(google___protobuf___message___Message):

    @property
    def identity(self) -> DeviceIdentity: ...

    def __init__(self,
        identity : typing___Optional[DeviceIdentity] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceGetIdentityResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class ResetIdsRequest(google___protobuf___message___Message):
    interface_id = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ResetIdsRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceIsBusyRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceIsBusyRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceIsBusyResponse(google___protobuf___message___Message):
    is_busy = ... # type: bool

    def __init__(self,
        is_busy : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceIsBusyResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class FirmwareVersion(google___protobuf___message___Message):
    major = ... # type: int
    minor = ... # type: int
    build = ... # type: int

    def __init__(self,
        major : typing___Optional[int] = None,
        minor : typing___Optional[int] = None,
        build : typing___Optional[int] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> FirmwareVersion: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceGetWarningsRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    clear = ... # type: bool

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        clear : typing___Optional[bool] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceGetWarningsRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceGetWarningsResponse(google___protobuf___message___Message):
    flags = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]

    def __init__(self,
        flags : typing___Optional[typing___Iterable[typing___Text]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceGetWarningsResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class SetLogOutputRequest(google___protobuf___message___Message):
    mode = ... # type: int
    file_path = ... # type: typing___Text

    def __init__(self,
        mode : typing___Optional[int] = None,
        file_path : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> SetLogOutputRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceGetSettingRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    setting = ... # type: typing___Text
    unit = ... # type: typing___Text

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        setting : typing___Optional[typing___Text] = None,
        unit : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceGetSettingRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceGetSettingResponse(google___protobuf___message___Message):
    value = ... # type: float

    def __init__(self,
        value : typing___Optional[float] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceGetSettingResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class DeviceSetSettingRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    setting = ... # type: typing___Text
    value = ... # type: float
    unit = ... # type: typing___Text

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        setting : typing___Optional[typing___Text] = None,
        value : typing___Optional[float] = None,
        unit : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> DeviceSetSettingRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class ToolsListSerialPortsResponse(google___protobuf___message___Message):
    ports = ... # type: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text]

    def __init__(self,
        ports : typing___Optional[typing___Iterable[typing___Text]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ToolsListSerialPortsResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class SetDeviceDbSourceRequest(google___protobuf___message___Message):
    source_type = ... # type: int
    url_or_file_path = ... # type: typing___Text

    def __init__(self,
        source_type : typing___Optional[int] = None,
        url_or_file_path : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> SetDeviceDbSourceRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class ToggleDeviceDbStoreRequest(google___protobuf___message___Message):
    toggle_on = ... # type: bool
    store_location = ... # type: typing___Text

    def __init__(self,
        toggle_on : typing___Optional[bool] = None,
        store_location : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> ToggleDeviceDbStoreRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class PrepareCommandRequest(google___protobuf___message___Message):
    interface_id = ... # type: int
    device = ... # type: int
    axis = ... # type: int
    command_template = ... # type: typing___Text

    @property
    def parameters(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[Measurement]: ...

    def __init__(self,
        interface_id : typing___Optional[int] = None,
        device : typing___Optional[int] = None,
        axis : typing___Optional[int] = None,
        command_template : typing___Optional[typing___Text] = None,
        parameters : typing___Optional[typing___Iterable[Measurement]] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> PrepareCommandRequest: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class PrepareCommandResponse(google___protobuf___message___Message):
    command = ... # type: typing___Text

    def __init__(self,
        command : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> PrepareCommandResponse: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...

class Measurement(google___protobuf___message___Message):
    value = ... # type: float
    unit = ... # type: typing___Text

    def __init__(self,
        value : typing___Optional[float] = None,
        unit : typing___Optional[typing___Text] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> Measurement: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...



import pytest
from concurrent.futures import Future

from zaber_motion.call import call, call_sync, call_async
from zaber_motion.events import events
from zaber_motion.protobufs import main_pb2
from zaber_motion.motion_lib_exception import MotionLibException
from zaber_motion.motion_lib_error_type import MotionLibErrorType


def test_request_response():
    request = main_pb2.TestRequest()
    request.data_ping = "Hello"

    response = main_pb2.TestResponse()
    call("test/request", request, response)

    assert response.data_pong == 'Hello...'

@pytest.mark.asyncio
async def test_request_response_async():
    request = main_pb2.TestRequest()
    request.data_ping = "Hello"

    response = main_pb2.TestResponse()
    await call_async("test/request", request, response)

    assert response.data_pong == 'Hello...'

def test_request_response_sync():
    request = main_pb2.TestRequest()
    request.data_ping = "Hello"

    response = main_pb2.TestResponse()
    call_sync("test/request", request, response)

    assert response.data_pong == 'Hello...'

def test_request_error():
    request = main_pb2.TestRequest()
    request.data_ping = "Hello"
    request.return_error = True

    try:
        response = main_pb2.TestResponse()
        call("test/request", request, response)
        assert False, "No error thrown"
    except MotionLibException as e:
        error_thrown = True
        assert e.error_type == MotionLibErrorType.REQUEST_TIMEOUT
        assert e.message == "Device has not responded in given timeout"


def test_request_no_response():
    response = call("test/request_no_response")
    assert response == None


def test_event():
    promise = Future()
    with events.take(1).subscribe(lambda event: promise.set_result(event)):
        call("test/emit_event")

        event = promise.result()
        assert event[0] == "test/event"
        assert event[1].data == "testing event data"
